@@ -2,8 +2,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -134,20 +132,29 @@ enum
 	kEHCIPortSC_TestControlPhase	= EHCIBitRangePhase(16, 19),
 	kEHCIPortSC_WKCNNT_E		= kEHCIBit20,
 	kEHCIPortSC_WKDSCNNT_E		= kEHCIBit21,
-
-	
 	kEHCILine_Low			= 1
 };
 
 
+// these are for the HCSPARAMS register
 enum
 {
-	kEHCINumPortsMask	= EHCIBitRange (0, 3),
-	kEHCIPPCMask 		= kEHCIBit4,	
-
-	kx = 0
+	kEHCINumPortsMask	= EHCIBitRange (0, 3),				// Number of ports (4 bits)
+	kEHCIPPCMask 		= kEHCIBit4,						// Power Port Control
 };
 
+
+// these are for the HCCPARAMS register
+enum
+{
+	kEHCIEECPMask		= EHCIBitRange (8, 15),				// EECP offset
+	kEHCIEECPPhase		= EHCIBitRangePhase(8, 15),			// EECP shift amount
+	kEHCI64Bit			= kEHCIBit0							// whether we use 64 bit addressing
+};
+
+
+
+// This is for various transfer desciptors 
 enum{
 	kEHCITermFlag = 1
 	};
@@ -588,5 +595,12 @@ enum
     kEHCIMicroFrameNumberIncrement	= kEHCIBit14		// ok, not really in the register, but related to the above
 };
 
+// bits in USBLEGSUP extended EHCI/PCI capability
+enum {
+    kEHCI_USBLEGSUP_ID        = 0x01,
+    kEHCI_USBLEGSUP_BIOSOwned = kEHCIBit16,
+    kEHCI_USBLEGSUP_OSOwned   = kEHCIBit24
+};
+
 #pragma mark ---end----
 #endif
