@@ -3,18 +3,19 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -24,6 +25,11 @@
 #ifndef _IOKIT_AppleUSBUHCI_H
 #define _IOKIT_AppleUSBUHCI_H
 
+#if DEBUG_LEVEL != DEBUG_LEVEL_PRODUCTION
+#define IOASSERT 1
+#include <IOKit/assert.h>
+#endif
+
 #include <libkern/OSByteOrder.h>
 
 extern "C" {
@@ -50,15 +56,17 @@ extern "C" {
 
 #if UHCI_USE_KPRINTF
 #undef USBLog
+#undef USBError
 void kprintf(const char *format, ...)
 __attribute__((format(printf, 1, 2)));
-#define USBLog( LEVEL, FORMAT, ARGS... )  if ((LEVEL) <= 2) { kprintf( FORMAT "\n", ## ARGS ) ; }
+#define USBLog( LEVEL, FORMAT, ARGS... )  if ((LEVEL) <= 5) { kprintf( FORMAT "\n", ## ARGS ) ; }
+#define USBError( LEVEL, FORMAT, ARGS... )  { kprintf( FORMAT "\n", ## ARGS ) ; }
 #endif
 
 #ifdef __ppc__
 #define IOSync eieio
 #else
-#define IOSync()
+#define IOSync() __asm__ __volatile__ ( "mfence" : : : "memory" )
 #endif
 
 /* Whether to use vertical TD queues, or fair queues. */
@@ -89,7 +97,8 @@ typedef struct AppleUHCIQH {
     IOPhysicalAddress   paddr;      /* Physical address of this structure. */
     int                 vframe;     /* Virtual frame location of this QH, if any. */
     struct AppleUHCIQH *hlink;
-    struct AppleUHCITD *elink;
+    struct AppleUHCITD *elink;      /* First TD */
+    struct AppleUHCIQH *qlink;      /* QH link */
 } QH;
 
 /* Software transaction descriptor structure.
@@ -199,10 +208,13 @@ typedef struct UHCIEndpoint {
     /* Support for isochronous endpoints. */
     TD **                               isoc_tds;
     
+    /* Interrupt chain used for interrupt endpoints. */
+    UInt32                              intr_queue;
+    
     TD *                                firstTD;      // Request queue.
     TD *                                lastTD;
     
-    queue_head_t                        pendingTransactions;
+    //queue_head_t                        pendingTransactions;
     queue_head_t                        activeTransactions; // Chain of active transactions
                                                       // for this endpoint.
 
@@ -340,6 +352,9 @@ protected:
     /* Copying buffers at interrupt time. */
     UHCIAlignmentBuffer * _interruptAlignmentBuffers;
 
+    /* Interrupt status bits. */
+    UInt32              _intrStatus;
+    
     /* Isochronous bandwidth management. */
     UInt32              _isocBandwidth;
 
@@ -359,20 +374,6 @@ protected:
     /* Interrupt queues. */
     QH * _intrQH[kUHCI_NINTR_QHS];
 
-    QH * FindPrevQH(QH *head, QH *qh);
-    
-    void QueueInterrupt(UHCITransaction *);
-    void RemoveInterrupt(UHCITransaction *);
-    
-    void QueueControl(UHCITransaction *);
-    void RemoveControl(UHCITransaction *);
-
-    void QueueBulk(UHCITransaction *);
-    void RemoveBulk(UHCITransaction *);
-    
-    void AddInterruptQH(QH *);
-    void RemoveInterruptQH(QH *);
-    
     /* Endpoint management. */
     queue_head_t _endpoints;
     
@@ -381,8 +382,9 @@ protected:
     queue_head_t _activeTransactions;
     
     void StartTransaction(UHCITransaction *tp);
-    void HWCompleteTransaction(UHCITransaction *tp);
+    void RemoveTransaction(UHCITransaction *tp);
     void CompleteTransaction(UHCITransaction *tp, IOReturn returnCode);
+    void FixEndpointDBits(UHCIEndpoint *ep);
 
     IOReturn TDToUSBError(UInt32 error);
     void CompleteIsoc(IOUSBIsocCompletion   completion,
@@ -424,6 +426,7 @@ protected:
     
     virtual IOReturn GetRootHubStringDescriptor(UInt8	index, OSData *desc);
 
+    void    StopEndpoint(UHCIEndpoint *ep);
     void    ReturnEndpointTransactions(UHCIEndpoint *ep, IOReturn status);
         
     /* UIM support. */
@@ -495,6 +498,7 @@ protected:
     void DumpQH(QH *, int level = 7);
     void DumpQHChain(QH *, int level = 7);
     void DumpFrame(UInt16 frame = 0, int level = 7);
+    void DumpEndpoint(UHCIEndpoint *ep, int level = 7);
     void SingleStep(int count, bool runAfter);
 
 
@@ -525,9 +529,13 @@ protected:
     UInt16 _saveFrameNumber;
     bool _remoteWakeupOccurred;
     int _powerLevel;
+    IONotifier *_powerDownNotifier;
+    
     virtual void initForPM (IOPCIDevice *provider);
     unsigned long maxCapabilityForDomainState ( IOPMPowerFlags domainState );
     unsigned long initialPowerStateForDomainState ( IOPMPowerFlags domainState );
+    static IOReturn 	PowerDownHandler(void *target, void *refCon, UInt32 messageType, IOService *service,
+                                         void *messageArgument, vm_size_t argSize);
     
     virtual IOReturn setPowerState( unsigned long powerStateOrdinal, IOService* whatDevice );
     
@@ -535,7 +543,7 @@ protected:
     void SuspendController(void);
     void StopController(void);
     void RestartController(void);
-    
+    void EnableUSBInterrupt(bool enableInterrupt);    
     
 public:
     virtual bool 	init(OSDictionary * propTable);
