@@ -2,8 +2,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -241,6 +239,12 @@ IOUSBCommand::SetTimeStamp(AbsoluteTime timeStamp)
     _expansionData->_timeStamp = timeStamp;
 }
 
+void
+IOUSBCommand::SetIsSyncTransfer(bool isSync)
+{
+    _expansionData->_isSyncTransfer = isSync;
+}
+
 
 usbCommand 
 IOUSBCommand::GetSelector(void) 
@@ -376,6 +380,11 @@ AbsoluteTime IOUSBCommand::GetTimeStamp(void)
     return _expansionData->_timeStamp;
 }
 
+bool IOUSBCommand::GetIsSyncTransfer(void)
+{
+    return _expansionData->_isSyncTransfer;
+}
+
 
 IOUSBIsocCommand*
 IOUSBIsocCommand::NewCommand()
