@@ -2,8 +2,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -23,9 +21,9 @@
  */
 
 
-#include <IOKit/usb/IOUSBLog.h>
+#include <sys/systm.h>
 
-#include </usr/include/stdio.h>
+#include <IOKit/usb/IOUSBLog.h>
 
 #ifdef	__cplusplus
 	extern "C" {
