@@ -38,9 +38,11 @@ class IOUSBNub : public IOService
     OSDeclareDefaultStructors(IOUSBNub)
 
 public:
-    virtual bool USBCompareProperty(OSDictionary   * matching,
+    virtual bool    USBCompareProperty(OSDictionary   * matching,
                                     const char     * key );
     
+    bool            IsWildCardMatch( OSDictionary   * matching, const char     * key );
+    bool	    USBComparePropertyWithMask( OSDictionary *matching, const char *key, const char * maskKey );
 };
 
 #ifdef __cplusplus
