@@ -1,16 +1,14 @@
 /*
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- *
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
  * compliance with the License. Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this
  * file.
- *
+ * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -18,7 +16,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- *
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -157,7 +155,7 @@ static void DeviceRemoved(void *refCon, io_iterator_t iterator)
     
     while (ioDeviceObj = IOIteratorNext(devIter)) {
         IOCFPlugInInterface 	**ioPlugin;
-        IOUSBDeviceInterface 	**deviceIntf = NULL;
+        IOUSBDeviceRef			deviceIntf = NULL;
         SInt32                  score;
         
         kr = IOCreatePlugInInterfaceForService(ioDeviceObj, kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID, &ioPlugin, &score);
@@ -189,7 +187,7 @@ static void DeviceRemoved(void *refCon, io_iterator_t iterator)
     [_listener busProberInformationDidChange:self];
 }
 
-- (void)processDevice:(IOUSBDeviceInterface **)deviceIntf deviceNumber:(int)deviceNumber {
+- (void)processDevice:(IOUSBDeviceRef)deviceIntf deviceNumber:(int)deviceNumber {
     BusProbeDevice *        thisDevice;
     UInt32                  locationID = 0;
     UInt8                   speed = 0;
@@ -197,6 +195,7 @@ static void DeviceRemoved(void *refCon, io_iterator_t iterator)
     IOUSBDeviceDescriptor   dev;
     int                     len;
     IOReturn                error;
+	BOOL					needToSuspend = FALSE;
 
     thisDevice = [[BusProbeDevice alloc] init];
     
@@ -225,14 +224,28 @@ static void DeviceRemoved(void *refCon, io_iterator_t iterator)
             locationID]];    
 
     error = GetDescriptor(deviceIntf, kUSBDeviceDesc, 0, &dev, sizeof(dev));
-    if (error == kIOReturnSuccess) {
+    if (error == kIOReturnNotResponding) 
+	{
+		// The device did not respond to a request for its device descriptor, probably because it was suspended.  Attempt to resume it and 
+		// later on suspend it again.
+		needToSuspend = TRUE;
+		
+		error = SuspendDevice(deviceIntf,false);
+		if ( error == kIOReturnSuccess )
+		{
+			error = GetDescriptor(deviceIntf, kUSBDeviceDesc, 0, &dev, sizeof(dev));
+		}
+	} 
+	
+	if ( error == kIOReturnSuccess )
+	{
         int iconfig;
-        [DecodeDeviceDescriptor decodeBytes:&dev forDevice:thisDevice deviceInterface:deviceIntf];
-
+        [DecodeDeviceDescriptor decodeBytes:&dev forDevice:thisDevice deviceInterface:deviceIntf wasSuspended:needToSuspend];
+		
         for (iconfig = 0; iconfig < dev.bNumConfigurations; ++iconfig) {
             IOUSBConfigurationDescHeader cfgHeader;
             IOUSBConfigurationDescriptor config;
-
+			
             // Get the Configuration descriptor.  We first get just the header and later we get the full
             // descriptor
             error = GetDescriptor(deviceIntf, kUSBConfDesc, iconfig, &cfgHeader, sizeof(cfgHeader));
@@ -260,12 +273,14 @@ static void DeviceRemoved(void *refCon, io_iterator_t iterator)
             }
             [DecodeConfigurationDescriptor decodeBytes:(IOUSBConfigurationDescHeader *)&cfgHeader forDevice:thisDevice deviceInterface:deviceIntf configNumber:iconfig isOtherSpeedDesc:NO];
         }
-    } else {
-        // The device did not respond to a request for its device descriptor
-        // This description will be shown in the UI, to the right of the device's name
-        [thisDevice setDeviceDescription: [NSString stringWithFormat:@"Unknown device (did not respond to inquiry - 0x%x)", error]];
     }
-    
+	else 
+	{
+		// This description will be shown in the UI, to the right of the device's name
+		[thisDevice setDeviceDescription: [NSString stringWithFormat:@"Unknown device (did not respond to inquiry - 0x%x)", error]];
+	}
+	
+	
     // If the device is a hub, then dump the Hub descriptor
     //
     if ( dev.bDeviceClass == kUSBHubClass )
@@ -327,6 +342,9 @@ static void DeviceRemoved(void *refCon, io_iterator_t iterator)
         }
     }
     
+	if ( needToSuspend )
+		error = SuspendDevice(deviceIntf,true);
+	
     [thisDevice release];
 }
 
