@@ -1,16 +1,14 @@
 /*
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- *
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
  * compliance with the License. Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this
  * file.
- *
+ * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -18,7 +16,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- *
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -28,8 +26,8 @@
 
 @implementation DecodeDeviceDescriptor
 
-+ (void)decodeBytes:(IOUSBDeviceDescriptor *)dev forDevice:(BusProbeDevice *)thisDevice deviceInterface:(IOUSBDeviceInterface **)deviceIntf {
-    NSString                *tempString1, *tempString2;
++ (void)decodeBytes:(IOUSBDeviceDescriptor *)dev forDevice:(BusProbeDevice *)thisDevice deviceInterface:(IOUSBDeviceRef)deviceIntf wasSuspended:(BOOL)wasSuspended {
+    NSString                *tempString1, *tempString2, *tempString3;
     BusProbeClass *         deviceClass;
     char                    *cstr1, *cstr2, *cstr3;
     
@@ -95,10 +93,15 @@
     tempString2 = [tempString1 stringByAppendingString:@" device: "];
     cstr2 = GetStringFromNumber(dev->idVendor, sizeof(dev->idVendor), kIntegerOutputStyle);
     
+	if ( wasSuspended )
+		tempString3 = [NSString stringWithFormat:@" (Suspended)"];
+	else
+		tempString3 = [NSString stringWithFormat:@""];
+
     if (strcmp(cstr1,"0x00") == 0) {
-        tempString2 = [NSString stringWithFormat:@"%@ device from %@", tempString1, VendorNameFromVendorID([NSString stringWithCString:cstr2])];
+        tempString2 = [NSString stringWithFormat:@"%@ device from %@%@", tempString1, VendorNameFromVendorID([NSString stringWithCString:cstr2]), tempString3];
     } else {
-        tempString2 = [NSString stringWithFormat:@"%@ device: %s", tempString1, cstr1];
+        tempString2 = [NSString stringWithFormat:@"%@ device: %s%@", tempString1, cstr1, tempString3];
     }
     
     FreeString(cstr1);
