@@ -1,16 +1,14 @@
 /*
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- *
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
  * compliance with the License. Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this
  * file.
- *
+ * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -18,7 +16,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- *
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -129,8 +127,23 @@
     
     [thisDevice addProperty:"Attributes:" withValue:str atDepth:ENDPOINT_LEVEL];
 
-    
-    sprintf(temporaryString, "%d", endpointDescriptor.wMaxPacketSize);
+	sprintf(temporaryString, "%d", endpointDescriptor.wMaxPacketSize);
+
+	if (useHighSpeedDefinition)
+    {
+        // If we have a USB 2.0 compliant device running at high speed, then the wMaxPacketSize calculation is funky
+        //
+        if ( interrupt || isoch)
+		{
+			UInt32	transPerMicroFrame = (endpointDescriptor.wMaxPacketSize & 0x1800) >> 11;
+			// If bits 15..13 are not 0, it's illegal.  If bits 12..11 are 11b, it's illegal
+            if ( ( (endpointDescriptor.wMaxPacketSize & 0xe000) != 0) || ( transPerMicroFrame == 3) )
+                sprintf(temporaryString, "0x%x: Illegal value for wMaxPacketSize for a hi-speed Interrupt endpoint", endpointDescriptor.bInterval);
+            else
+                sprintf(temporaryString, "%d  (%d x %ld  transactions opportunities per microframe)", endpointDescriptor.wMaxPacketSize, endpointDescriptor.wMaxPacketSize & 0x07ff,  transPerMicroFrame + 1);
+        }
+    }
+	
     [thisDevice addProperty:"Max Packet Size:" withValue:temporaryString atDepth:ENDPOINT_LEVEL];
     
     if (useHighSpeedDefinition)
@@ -149,13 +162,15 @@
             if ( (endpointDescriptor.bInterval == 0) || (endpointDescriptor.bInterval > 16) )
                 sprintf(temporaryString, "%d: Illegal value for bInterval for a hi-speed Interrupt endpoint", endpointDescriptor.bInterval);
             else
-                sprintf(temporaryString, "%d ( %d microframes (%d ms) )", endpointDescriptor.bInterval, (1 << (endpointDescriptor.bInterval-1)), (1 << (endpointDescriptor.bInterval-4)));
+                sprintf(temporaryString, "%d (%d %s (%d %s) )", endpointDescriptor.bInterval, (1 << (endpointDescriptor.bInterval-1)), endpointDescriptor.bInterval==1?"microframe":"microframes", 
+						(endpointDescriptor.bInterval > 3 ? (1 << (endpointDescriptor.bInterval-1))/8 : endpointDescriptor.bInterval * 125), endpointDescriptor.bInterval > 3?"msecs":"microsecs" );
         
         if ( isoch )
             if ( (endpointDescriptor.bInterval == 0) || (endpointDescriptor.bInterval > 16) )
                 sprintf(temporaryString, "Illegal value for bInterval for a hi-speed isoch endpoint: %d", endpointDescriptor.bInterval);
             else
-                sprintf(temporaryString, "%d ( %d microframes (%d ms) )", endpointDescriptor.bInterval, (1 << (endpointDescriptor.bInterval-1)), (1 << (endpointDescriptor.bInterval-4)));
+                sprintf(temporaryString, "%d (%d %s (%d %s) )", endpointDescriptor.bInterval, (1 << (endpointDescriptor.bInterval-1)), endpointDescriptor.bInterval==1?"microframe":"microframes", 
+						endpointDescriptor.bInterval > 3 ? (1 << (endpointDescriptor.bInterval-1))/8 : endpointDescriptor.bInterval * 125, endpointDescriptor.bInterval > 3?"msecs":"microsecs" );
     }
     else
     {
