@@ -1,16 +1,14 @@
 /*
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- *
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
  * compliance with the License. Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this
  * file.
- *
+ * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -18,7 +16,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- *
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -28,7 +26,7 @@
 
 @implementation DecodeHIDDescriptor
 
-+ (void)decodeBytes:(Byte *)p forDevice:(BusProbeDevice *)thisDevice withDeviceInterface:(IOUSBDeviceInterface **)deviceIntf {
++ (void)decodeBytes:(Byte *)p forDevice:(BusProbeDevice *)thisDevice withDeviceInterface:(IOUSBDeviceRef)deviceIntf {
     char 	temporaryString[500];
     BusProbeClass *     lastInterfaceClassInfo = [thisDevice lastInterfaceClassInfo];
     
@@ -145,7 +143,7 @@
     UInt32			usagePage = 0;
     UInt32			value=0;
     SInt32			svalue=0;
-    static unsigned char	buf[350], tempbuf[350], bufvalue[350], tempbufvalue[350];
+    static  char	buf[350], tempbuf[350], bufvalue[350], tempbufvalue[350];
     int				i, indentLevel;
     Boolean			datahandled=false;
     Boolean			usagesigned=false;
