@@ -1,16 +1,14 @@
 /*
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- *
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
  * compliance with the License. Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this
  * file.
- *
+ * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -18,7 +16,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- *
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -30,13 +28,13 @@
 
 - initWithName:(NSString *)name value:(NSString *)value {
     if (self = [super initWithName:name value:value]) {
-        _representedDevice = NULL;
+        _representedDevice = nil;
     }
     return self;
 }
 
 - (void)dealloc {
-    if (_representedDevice != NULL) {
+    if (_representedDevice != nil) {
         IOObjectRelease(_representedDevice);
     }
     [super dealloc];
@@ -47,7 +45,7 @@
 }
 
 - (void)setRepresentedDevice:(io_object_t)object {
-    if (_representedDevice != NULL) {
+    if (_representedDevice != nil) {
         IOObjectRelease(_representedDevice);
     }
     IOObjectRetain(object);
