@@ -1,16 +1,14 @@
 /*
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * Copyright (c) 1998-2003 Apple Computer, Inc.  All Rights Reserved.
- *
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
  * compliance with the License. Please obtain a copy of the License at
  * http://www.opensource.apple.com/apsl/ and read it before using this
  * file.
- *
+ * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
@@ -18,7 +16,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
  * Please see the License for the specific language governing rights and
  * limitations under the License.
- *
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -74,6 +72,12 @@ enum
     kKernelDebugOutputKextLoggerType	= 0x00000002
 };
 
+@protocol USBLoggerListener <NSObject>
+
+- (void)usbLoggerTextAvailable:(NSString *)text forLevel:(int)level;
+
+@end
+
 @interface USBLogger : NSObject {
     id              _listener;
     int             _loggingLevel;
@@ -85,7 +89,7 @@ enum
     IODataQueueMemory *	_gMyQueue;
 }
 
-- initWithListener:(id)listener level:(int)level;
+- initWithListener:(id <USBLoggerListener>)listener level:(int)level;
 
 - (kern_return_t)OpenUSBControllerUserClient;
 - (kern_return_t)setDebuggerOptions:(int)shouldLogFlag setLevel:(bool)setLevel level:(UInt32)level setType:(bool)setType type:(UInt32)type;
@@ -96,9 +100,4 @@ enum
 
 @end
 
-@protocol USBLoggerListener <NSObject>
-
-- (void)usbLoggerTextAvailable:(NSString *)text forLevel:(int)level;
-
-@end
 
